<?php

namespace HCM\Util\NewFileServer\FileServerClasses\FSMigration\FileServer\Files;

use HCM\Util\NewFileServer\FileServerClasses\FSMigration\FileServer\Files\IFSFilesClient;
use HCM\Util\NewFileServer\FileServerClasses\FSMigration\FileServer\APIKeyHandler;
use HCM\Util\NewFileServer\FileServerClasses\Util\MyCurl;

class FSFilesClient extends APIKeyHandler implements IFSFilesClient {
    const FS_DEFAULT_BASE_URL       = "http://192.168.120.130:8083/zeedhi/fileserver/teste/backend/service/index.php";

    const FS_DELETE_ROUTE           = "/delete";
    const FS_UPLOAD_ROUTE           = "/file/upload";
    const FS_LIST_FILES_ROUTE       = "/dump";
    const FS_GET_FILE_ROUTE         = "/files/";

    private $fileServerBaseUrl;

    /***
     *
     * $nrorg -> A number or a string that holds the API Key Organization ID
     *
     * $APIKeyToken -> Organization API Key
     *
     * $fileServerBaseUrl -> A URL poiting to FileServer's Base WebService Path.
     *
     ***/
    public function __construct($nrorg, $APIKeyToken, $fileServerBaseUrl = self::FS_DEFAULT_BASE_URL) {
        parent::__construct($APIKeyToken);
        $this->nrorg = $nrorg;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
    }

    public function getNrorg() {
        return $this->nrorg;
    }

    /**
    * $fileData -> An array with fileServer's following format:
    *   {
    *       "name": "yourFileName"
    *       "type": "yourMimeType",
    *       "size": 1024, //the size of your file
    *       "path": "the/path/of/your/file",
    *       "b64File": "data:mime/type;base64,yourBase64EncodedFile"
    *   }
    *
    */
    public function uploadFile(array $fileData) {
        $url = $this->fileServerBaseUrl . self::FS_UPLOAD_ROUTE;
        $headers = $this->getFileServerCustomHeader();

        $folderName = $fileData['folderName'];
        unset($fileData['folderName']);

        $params = array(
            "requestType" => "Row",
            "row" => array(
                "FILE" =>  $fileData,
                "FOLDER" => $folderName
            )
        );

        try {
            $response = MyCurl::post($url, $params, $headers);
        } catch (\Exception $e) {
            throw new \Exception('(FILESERVER) ' . $e->getMessage(), $e->getCode());
        }

        $path = $response['dataset']['files'][0]['PATH'];

        return $path;
    }

    /**
    * $filesData -> An array with fileServer's following format:
    *   [
    *       {
    *           "name": "yourFileName"
    *           "type": "yourMimeType",
    *           "size": 1024, //the size of your file
    *           "path": "the/path/of/your/file",
    *           "b64File": "data:mime/type;base64,yourBase64EncodedFile"
    *       }
    *   ]
    *
    */

    public function uploadFiles(array $filesData) {
        $url = $this->fileServerBaseUrl . self::FS_UPLOAD_ROUTE;
        $headers = $this->getFileServerCustomHeader();


        $zeedhiFileServerObj = array(
            "dataset" => $filesData,
            "requestType" => "DataSet"
        );

        $result = MyCurl::post($url, $zeedhiFileServerObj, $headers);
        return $result['files'];
    }

    /**
    * Requests the FileServer the file with the respective path.
    * @param string $fileOnServerPath A string containg the path to the file once stored at FileServer.
    * @return array Array with the file in base64 format and its MIME type.
    */
    public function getFile(string $fileOnServerPath): array {
        $headers = $this->getFileServerCustomHeader();
        $url = $this->fileServerBaseUrl . self::FS_GET_FILE_ROUTE . $fileOnServerPath;

        try {
            $response = MyCurl::get($url, $headers, false);
        } catch (\Exception $e) {
            throw new \Exception('(FILESERVER) ' . $e->getMessage(), $e->getCode());
        }

        $base64 = base64_encode($response);

        $f = finfo_open();
        $mime_type = finfo_buffer($f, $response, FILEINFO_MIME_TYPE);

        $file = [
            'type'   => $mime_type,
            'base64' => $base64
        ];

        return $file;
    }

    /**
    * Deletes the file with the respective path.
    * @param string $path A string containg the path to the file once stored at FileServer.
    * @return bool Says if the file was deleted.
    */
    public function deleteFile(string $path): bool {
        $headers = $this->getFileServerCustomHeader();
        $url = $this->fileServerBaseUrl . self::FS_DELETE_ROUTE;


        $params = array(
            "requestType" => "Row",
            "row" => [
                "PATH" => $path
            ]
        );

        $response = MyCurl::post($url, $params, $headers);
        $qtDeleted = $response['dataset']['file']['deleted'];

        $deleted = $qtDeleted > 0;

        return $deleted;
    }

    public function listFiles() {
        $headers = $this->getFileServerCustomHeader();
        $url = $this->fileServerBaseUrl . self::FS_LIST_FILES_ROUTE;
        return MyCurl::get($url, $headers);
    }
}